package sun.java2d.pipe;

import sun.java2d.SurfaceData;
import java.awt.image.Raster;
import java.awt.CompositeContext;
import sun.java2d.loops.Blit;
import java.awt.AlphaComposite;
import sun.java2d.loops.MaskBlit;
import sun.java2d.loops.CompositeType;
import sun.awt.image.BufImgSurfaceData;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.awt.PaintContext;
import java.awt.image.ColorModel;
import java.awt.RenderingHints;
import java.awt.Rectangle;
import java.awt.Shape;
import sun.java2d.SunGraphics2D;

public class GeneralCompositePipe implements CompositePipe
{
    public GeneralCompositePipe() {
    }
    
    @Override
    public Object startSequence(SunGraphics2D sunGraphics2D, Shape shape, Rectangle rectangle, int[] array) {
        RenderingHints renderingHints = sunGraphics2D.getRenderingHints();
        ColorModel deviceColorModel = sunGraphics2D.getDeviceColorModel();
        PaintContext context = sunGraphics2D.paint.createContext(deviceColorModel, rectangle, shape.getBounds2D(), sunGraphics2D.cloneTransform(), renderingHints);
        return new GeneralCompositePipe.TileContext(sunGraphics2D, context, sunGraphics2D.composite.createContext(context.getColorModel(), deviceColorModel, renderingHints), deviceColorModel);
    }
    
    @Override
    public boolean needTile(Object o, int n, int n2, int n3, int n4) {
        return true;
    }
    
    @Override
    public void renderPathTile(Object o, byte[] array, int n, int n2, int x, int y, int n3, int n4) {
        GeneralCompositePipe.TileContext tileContext = (GeneralCompositePipe.TileContext)o;
        PaintContext paintCtxt = tileContext.paintCtxt;
        CompositeContext compCtxt = tileContext.compCtxt;
        SunGraphics2D sunG2D = tileContext.sunG2D;
        Raster raster = paintCtxt.getRaster(x, y, n3, n4);
        paintCtxt.getColorModel();
        SurfaceData surfaceData = sunG2D.getSurfaceData();
        Raster raster2 = surfaceData.getRaster(x, y, n3, n4);
        WritableRaster writableRaster;
        if (raster2 instanceof WritableRaster && array == null) {
            writableRaster = (writableRaster = ((WritableRaster)raster2).createWritableChild(x, y, n3, n4, 0, 0, (int[])null));
        }
        else {
            writableRaster = raster2.createChild(x, y, n3, n4, 0, 0, (int[])null);
            writableRaster = writableRaster.createCompatibleWritableRaster();
        }
        compCtxt.compose(raster, writableRaster, writableRaster);
        if (raster2 != writableRaster && writableRaster.getParent() != raster2) {
            if (raster2 instanceof WritableRaster && array == null) {
                ((WritableRaster)raster2).setDataElements(x, y, writableRaster);
            }
            else {
                ColorModel deviceColorModel = sunG2D.getDeviceColorModel();
                SurfaceData data = BufImgSurfaceData.createData(new BufferedImage(deviceColorModel, writableRaster, deviceColorModel.isAlphaPremultiplied(), null));
                if (array == null) {
                    Blit.getFromCache(data.getSurfaceType(), CompositeType.SrcNoEa, surfaceData.getSurfaceType()).Blit(data, surfaceData, AlphaComposite.Src, null, 0, 0, x, y, n3, n4);
                }
                else {
                    MaskBlit.getFromCache(data.getSurfaceType(), CompositeType.SrcNoEa, surfaceData.getSurfaceType()).MaskBlit(data, surfaceData, AlphaComposite.Src, null, 0, 0, x, y, n3, n4, array, n, n2);
                }
            }
        }
    }
    
    @Override
    public void skipTile(Object o, int n, int n2) {
    }
    
    @Override
    public void endSequence(Object o) {
        GeneralCompositePipe.TileContext tileContext = (GeneralCompositePipe.TileContext)o;
        if (tileContext.paintCtxt != null) {
            tileContext.paintCtxt.dispose();
        }
        if (tileContext.compCtxt != null) {
            tileContext.compCtxt.dispose();
        }
    }
    
    class TileContext
    {
        SunGraphics2D sunG2D;
        PaintContext paintCtxt;
        CompositeContext compCtxt;
        ColorModel compModel;
        Object pipeState;
        
        public TileContext(SunGraphics2D sunG2D, PaintContext paintCtxt, CompositeContext compCtxt, ColorModel compModel) {
            this.sunG2D = sunG2D;
            this.paintCtxt = paintCtxt;
            this.compCtxt = compCtxt;
            this.compModel = compModel;
        }
    }
}
